"use strict";

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.RBFormObjectRecursiveForEach = void 0;

var _react = _interopRequireDefault(require("react"));

var _Field = require("./Field");

var _FieldArray = require("./FieldArray");

var _Validation = require("./Validation");

var _Utils = require("./Utils");

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

var getValidValue = (value, _ref) => {
  var {
    type,
    multiple,
    htmlSize,
    dateFormat,
    timeFormat,
    locale
  } = _ref;

  if (type === 'datetime') {
    return (0, _Utils.getDatetimeString)(value, (0, _Utils.getDatetimeFormat)(dateFormat, timeFormat), locale);
  }

  if (type === 'select' && multiple || htmlSize) {
    return (0, _Utils.isArray)(value) ? value : [];
  }

  if (type === 'color' && value === '') {
    return '#000000';
  }

  if (type === 'checkbox' || type === 'switch') {
    return !!value;
  }

  return value;
};

var getInitialValue = (name, value, initialValuesForm, defaultValue, props, index) => {
  // Priority 1: Field specific value.
  if (value != null) {
    return getValidValue(value, props);
  } // Priority 2: Form initial value.
  // This object has been constructed from the start and the algorithm is keeping track of the remaing depth with each array.
  // So the next correct key is inside the array of objects with the current index.


  if (initialValuesForm != null) {
    if ((0, _Utils.isArray)(initialValuesForm) && index != null) {
      var initialValue = initialValuesForm[index] && initialValuesForm[index][name];

      if (initialValue) {
        return getValidValue(initialValue, props);
      }
    } else if (initialValuesForm[name] != null) {
      return getValidValue(initialValuesForm[name], props);
    }
  } // Priority 3: Field specific default value.


  if (defaultValue != null) {
    return getValidValue(defaultValue, props);
  }

  return getValidValue('', props);
};

var getDefaultValue = (value, defaultValue, props) => {
  if (value != null) {
    return getValidValue(value, props);
  }

  if (defaultValue != null) {
    return getValidValue(defaultValue, props);
  }

  return getValidValue('', props);
};

var RBFormObjectRecursiveForEach = (validationShape, valuesHidden, defaultValues, initialValuesForm, initialValues, children) => {
  children.forEach(child => {
    if (! /*#__PURE__*/_react.default.isValidElement(child)) {
      return;
    }

    var {
      name,
      disabled,
      hidden,
      defaultValue,
      value,
      initialSize,
      customInput,
      children
    } = child.props; // Used when there is a FormArray inside another FormArray.

    var indexInner = null;
    var initialValuesFormInner = null;
    var initialValuesInner = null;
    var defaultValuesInner = null; // Ensures that the possible next node has the correct references.

    var validationShapeParent = validationShape;
    var valuesHiddenParent = valuesHidden;
    var defaultValuesParent = defaultValues;
    var initialValuesFormParent = initialValuesForm;
    var initialValuesParent = initialValues;
    var typeComponent = customInput ? _Field.RBField : child.type;

    switch (typeComponent) {
      case _Field.RBField:
        {
          validationShape[name] = (0, _Validation.getShape)(child.props);
          valuesHidden[name] = disabled || hidden; // Get default values.

          if ((0, _Utils.isArray)(defaultValues)) {
            defaultValues.forEach(object => {
              object[name] = getDefaultValue(value, defaultValue, child.props);
            });
          } else {
            defaultValues[name] = getDefaultValue(value, defaultValue, child.props);
          } // Get initial values.


          if ((0, _Utils.isArray)(initialValues)) {
            initialValues.forEach((object, index) => {
              object[name] = getInitialValue(name, value, initialValuesForm, defaultValue, child.props, index);
            });
          } else {
            initialValues[name] = getInitialValue(name, value, initialValuesForm, defaultValue, child.props);
          }

          break;
        }

      case _FieldArray.RBFieldArray:
        {
          var sizeMin = Number.isInteger(initialSize) ? initialSize : 1; // These will be filled by the underlying Fields.

          validationShapeParent = {};
          valuesHiddenParent = {};

          if ((0, _Utils.isArray)(initialValuesForm)) {
            // Previous node was a Field Array, so the initialValuesForm is an array with objects.
            // Each object in the array must hold an array with the current key name, for both initialValuesForm and initialValues.
            // Note: The previous Field Array made sure that initialValuesForm has the correct size.
            indexInner = 0;
            initialValuesInner = []; // Setup default value.

            if ((0, _Utils.isArray)(defaultValues)) {
              defaultValuesInner = defaultValues.map(value => {
                value[name] = (0, _Utils.createArray)(sizeMin, {});
                return value;
              });
              defaultValuesParent = defaultValuesInner[indexInner][name];
            } else {
              defaultValuesParent = (0, _Utils.createArray)(sizeMin, {});
            } // Iterate initialValuesForm to get the initial value of the current key name.


            initialValuesFormInner = initialValuesForm.map(value => {
              var dataKey = value[name]; // Element must be an array.

              if ((0, _Utils.isArray)(dataKey)) {
                dataKey = (0, _Utils.fillMissingArrayRows)(sizeMin, dataKey, []);
                initialValuesInner.push((0, _Utils.createArray)(dataKey.length, {}));
                return dataKey;
              }

              initialValuesInner.push((0, _Utils.createArray)(sizeMin, {}));
              return (0, _Utils.createArray)(sizeMin, []);
            });
            initialValuesFormParent = initialValuesFormInner[indexInner];
            initialValuesParent = initialValuesInner[indexInner];
          } else {
            var _initialValuesForm$na;

            // Previous node wasn't a Field Array, so the initialValuesForm is an object.
            // One element in both initialValuesForm and initialValues must hold an array with the current key name.
            // These arrays will hold object that will be filled by underlying nodes.
            // The correct length of the arrays must also be predetermined.
            // This ensures that the next nodes know how much rows there should be.
            // Note: The props initialValues doesn't have any valid data, since it should be filled by the Fields.
            defaultValuesParent = {}; // Get the array of objects with the initial values.

            initialValuesFormParent = (_initialValuesForm$na = initialValuesForm[name]) !== null && _initialValuesForm$na !== void 0 ? _initialValuesForm$na : [];
            var sizeInitialValuesFormParent = initialValuesFormParent.length; // Determine the amount of rows. InitialValues(Form) and initialSize are the only props that can influence this.

            var rowsAmount = Math.max(sizeInitialValuesFormParent, sizeMin);
            initialValuesFormParent = (0, _Utils.fillMissingArrayRows)(rowsAmount, initialValuesFormParent);
            initialValuesParent = (0, _Utils.createArray)(rowsAmount, {});
          }

          break;
        }

      default:
        break;
    }

    if (children) {
      RBFormObjectRecursiveForEach(validationShapeParent, valuesHiddenParent, defaultValuesParent, initialValuesFormParent, initialValuesParent, (0, _Utils.ensureArray)(children));

      if (child.type === _FieldArray.RBFieldArray) {
        if (initialValuesInner) {
          var initialValuesParentTemp = [initialValuesParent];
          var defaultValuesParentTemp = [defaultValuesParent];

          while (++indexInner < initialValuesInner.length) {
            initialValuesFormParent = initialValuesFormInner[indexInner];
            initialValuesParent = initialValuesInner[indexInner];

            if (defaultValuesInner) {
              defaultValuesParent = defaultValuesInner[indexInner][name];
            }

            RBFormObjectRecursiveForEach(validationShapeParent, valuesHiddenParent, defaultValuesParent, initialValuesFormParent, initialValuesParent, (0, _Utils.ensureArray)(children));
            initialValuesParentTemp.push(initialValuesParent);
            defaultValuesParentTemp.push(defaultValuesParent);
          }

          initialValuesParent = initialValuesParentTemp;

          if (defaultValuesInner) {
            defaultValuesParent = defaultValuesParentTemp;
          }
        }

        validationShape[name] = (0, _Validation.getShapeArray)(validationShapeParent);
        valuesHidden[name] = valuesHiddenParent;
        defaultValues[name] = defaultValuesParent;

        if ((0, _Utils.isArray)(initialValues)) {
          initialValues.forEach((object, index) => {
            if ((0, _Utils.isArray)(initialValuesParent)) {
              object[name] = initialValuesParent[index];
            } else {
              object[name] = initialValuesParent;
            }
          });
        } else {
          initialValues[name] = initialValuesParent;
        }
      }
    }
  });
};

exports.RBFormObjectRecursiveForEach = RBFormObjectRecursiveForEach;